﻿-- Daily Global Check - Garrison Follower Missions plugin
-- Jadya EU-Well of Eternity
local addonName, addonTable = ...

-- constants
local pref_achiinfo = "#achiinfo#"
local pref_achicrit = "#achicrit#"
local pref_itemname = "#item#"
local foreach = table.foreach

local pID
local pTable
local questsdata
DGC_GarrMLocalizedStrings = {}

local plugintitle = "Garrison Missions"
local pluginicon = "Interface\\Icons\\Achievement_General_CityAttacker2"

-- table orders
local CENTER, LEFT, RIGHT = 1,2,3

local function formattime(s)
 return string.format("%.2d:%.2d:%.2d", s/(60*60), s/60%60, s%60)
end

local function isquestcompleted(questID)
 local sr = "No Missions"
 local c = false
 local minor = 999999999999999
 local now = time()
 local counter, ready = 0,0
 foreach(pTable[questID].timers, function(k,v)
  if v < minor then
   c = now > v
   sr = c and "Ready" or formattime(v - now)
   minor = v
  end
  if now > v then ready = ready + 1 end
  counter = counter + 1
 end)
 if c then
  sr = ready.." of "..counter.." Ready";
 end
 return c, sr
end

local plugin_data = {
 ["Title"] = plugintitle,
 ["Icon"]  = pluginicon,
 ["Data"]  = nil,
 ["Order"] = nil,
 ["Overrides"] = {["isquestcompleted"] = isquestcompleted}
 }

-- template = {[ZONE],[NAME],[PREF],[SUFF],{{x,y},{x,y}},defaultmapID,[QUESTTYPE],[MAPICON],showfunc
local function GenerateQuestsData()
 if not DailyGlobalCheck then return end
 questsdata = {}
 
 local function gs(v)
  return v == nil and "" or v
 end

 plugin_data["Data"] = questsdata
 plugin_data["Order"] = {}
 plugin_data["Order"][CENTER] = {{"List"}}
 
 foreach(pTable, function(k,v)
  if v.show then
   questsdata[k] = {gs(v.realm),gs(v.name),"",""},
   table.insert(plugin_data["Order"][CENTER][1], k)
  end
 end)
 DailyGlobalCheck:SetFramesData(plugin_data["Data"], plugin_data["Order"], plugintitle)
end

local function Initialize()
 if not DailyGlobalCheck then return end
 DailyGlobalCheck:LoadPlugin(plugin_data)
end

local function checkvisibility()
 local found
 foreach(pTable[pID].timers, function(k,v)
  found = true
 end)
 
 if found then return end
 
 -- no timers available
 pTable[pID].show = nil
 GenerateQuestsData()
end

local function removeunusedmissions()
 foreach(pTable[pID].timers, function(mID, t)
   local found
   foreach(C_Garrison.GetInProgressMissions(), function(k,v)
    if mID == v.missionID then found = true end
   end)
   foreach(C_Garrison.GetCompleteMissions(), function(k,v)
    if mID == v.missionID then found = true end
   end)
   if not found then
    pTable[pID].timers[mID] = nil
   end
  end)
  checkvisibility()
end

local initialized = false
local eventframe = CreateFrame("FRAME")
eventframe:RegisterEvent("ADDON_LOADED")
eventframe:RegisterEvent("PLAYER_ENTERING_WORLD")
eventframe:RegisterEvent("GARRISON_MISSION_STARTED")
eventframe:RegisterEvent("GARRISON_MISSION_COMPLETED")
local function eventhandler(self, event, ...)
 if event == "ADDON_LOADED" and ... == addonName then
  local pName = GetUnitName("player")
  local pRealm = GetRealmName()
  pID = pName.."-"..pRealm
  if not DGC_GarrMLocalizedStrings[pID] then DGC_GarrMLocalizedStrings[pID] = {} end
  
  pTable = DGC_GarrMLocalizedStrings
  if not pTable[pID].timers then pTable[pID].timers = {} end
  pTable[pID].name = pName
  pTable[pID].realm = pRealm
 
  GenerateQuestsData()
  Initialize()
  initialized = true
 elseif event == "GARRISON_MISSION_COMPLETED" then
  local missionID = ...
  local t = time()
  if pTable[pID].timers[missionID] then
   pTable[pID].timers[missionID] = nil
  end
 removeunusedmissions()
 elseif event == "GARRISON_MISSION_STARTED" then
  local missionID = ...
  local _, st, _, _, et = C_Garrison.GetMissionTimes(missionID)
  local t = time() + st + et
  pTable[pID].timers[missionID] = t
  pTable[pID].show = true
  GenerateQuestsData()
  removeunusedmissions()
 elseif event == "PLAYER_ENTERING_WORLD" then
  if not initialized then Initialize() end
  eventframe:UnregisterEvent("PLAYER_ENTERING_WORLD")
 end
end
eventframe:SetScript("OnEvent", eventhandler)